from __future__ import annotations


# 导入module避免某些循环import的问题
from dknovautils import commons, iprint_debug, iprint_warn, AT  # type:ignore
from dknovautils.commons import *


class DKProcessUtil(object):
    _cmdid = 1000

    _retcode_default: int = -999

    @staticmethod
    def run_simple_a(cmd: str, verbose: bool = False) -> int:
        """执行系统命令并有进程返回码。
        
        这个例子说明 call() 函数具有不可替代的价值。


        """
        cmdid = DKProcessUtil._cmdid
        DKProcessUtil._cmdid += 1

        if verbose:
            iprint_debug(f"run_simple_a {cmdid} begin {cmd[:1024]}")

        retcode: int = DKProcessUtil._retcode_default

        try:
            retcode = subprocess.call(cmd, shell=True)
            if verbose and retcode < 0:
                iprint_debug(f"Child was terminated by signal {retcode}")

            if verbose:
                iprint_debug(f"Child returned {retcode}")
        except OSError as e:
            iprint_warn(f"run_simple_a {cmdid} Execution failed: {e}")
        finally:
            if verbose:
                iprint_debug(f"run_simple_a {cmdid} end ")

        return retcode

    @staticmethod
    def run_simple_b(
        cmd: str,
        *,
        splitLines: bool = False,
        throwOnErrReturnCode: bool = False,
        strip: bool = True,
        verbose: bool = False,
        encoding: Any = None,
        errors: Any = None,
    ) -> Tuple[int, str, str]:
        """ 
        
        Use shell to execute the command, store the stdout and stderr in sp variable
        
        """
        # Tuple[int, str|List,str|List]
        cmdid = DKProcessUtil._cmdid
        DKProcessUtil._cmdid += 1

        AT.assert_(not splitLines, "err51042 no splitlines")

        if verbose:
            iprint_debug(f"run_simple_b {cmdid} start {cmd[:1024]}")

        # Use shell to execute the command, store the stdout and stderr in sp variable
        # 运行中，会缓存结果，直到执行完成才会输出。
        sp = subprocess.Popen(
            cmd,
            shell=True,
            universal_newlines=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            encoding=encoding,
            errors=errors,
        )

        # Store the return code in rc variable
        # 这个rc值如何获取？如果打开这一行，将在某些情况下产生死锁问题。
        # 来自文档中的说明: This will deadlock when using stdout=PIPE or stderr=PIPE and the child process generates enough output to a pipe such that it blocks waiting for the OS pipe buffer to accept more data. Use Popen.communicate() when using pipes to avoid that.
        # rc = sp.wait()

        # Separate the output and error by communicating with sp variable.
        # This is similar to Tuple where we store two values to two different variables
        out, err = sp.communicate()
        rc: int | None = sp.returncode
        rc = rc if rc else 0

        if verbose:
            iprint_debug(f"Return Code: {rc}")
            iprint_debug(f"stdout is: {out[:1024]}")
            if rc != 0:
                iprint_debug(f"stderr is: {err}")

        if verbose:
            iprint_debug(f"run_simple_b {cmdid} end ")

        if out is None:
            out = ""
        if err is None:
            err = ""

        if strip:
            out = out.strip()
            err = err.strip()

        if throwOnErrReturnCode and (rc != 0):
            msg = f"cmd {cmdid} run error. err: {rc} {err}"
            raise Exception(msg)

        """
        
        这个函数发现的问题之一 是在wsl中执行robocopy等win命令时 会输出非unicode编码的字符 从而导致out解码出现错误.
        有时候似乎加上encoding也不行. 比如 encoding='utf-16'也是不行的
        这个问题后来似乎解决了？忘记是怎么解决的了？
        
        
        关于encoding errors 参考如下文档
        https://docs.python.org/3/howto/unicode.html
        https://docs.python.org/3/library/io.html#io.TextIOWrapper
        
        
        
        """

        return (rc, out, err)


run_simple_a = DKProcessUtil.run_simple_a
run_simple_b = DKProcessUtil.run_simple_b


"""




"""



from dknovautils import commons, iprint_debug, iprint_warn, AT  # type:ignore
from dknovautils.commons import *
import socket


class DkNetwork:

    @classmethod
    def http_get_url_simple_str(
        cls,
        url: str,
        encoding: str = "utf-8",
        timeout: float = 5.0,
        verbose: bool = False,
    ) -> str | None:
        """

        https://stackabuse.com/guide-to-sending-http-requests-in-python-with-urllib3/

        """
        _http = urllib3.PoolManager(timeout=timeout)

        try:
            if verbose:
                iprint_debug(f"get url start {url}")
            r = _http.request("GET", url, timeout=timeout, retries=False)
            # print(json.loads(r.data.decode('utf-8')))
            s = r.data.decode(encoding)
            assert isinstance(s, str)
            return s
        except Exception as e:
            iprint_debug(f"err65715 {url} {e}")
            return None
        finally:
            if verbose:
                iprint_debug(f"get url end {url}")




http_get_url_simple_str = DkNetwork.http_get_url_simple_str


def _f_test_http_get_url_simple_str() -> None:
    url = "https://www.baidu.com/"
    r = http_get_url_simple_str(url)
    assert r and len(r) > 10

    url = "http://www.baidu.com/"
    r = http_get_url_simple_str(url)
    assert r and len(r) > 10

    url = "http://192.168.0.1/"
    r = http_get_url_simple_str(url)
    print(r)
    assert r and len(r) > 10

    url = "http://192.168.0.2/"
    r = http_get_url_simple_str(url, verbose=True)
    print(r)
    assert r is None

    print("test end")


def http_scan_urls(
    urls: List[str], *, limit: int = 200, timeout: float = 5.0
) -> List[str]:
    """
    顺序返回能正常获取内容的url list。
    limit是限制打印的对应的值的文本长度。没有多大作用。
    实现中是多线程访问网络。

    """
    # port = 9310
    # urls = [f"http://192.168.0.{i}:{port}" for i in range(1, 256)]

    ma = {}

    def fa(url: str) -> None:
        ma[url] = None
        rs = http_get_url_simple_str(url, timeout=timeout)
        ma[url] = rs

    ths = [threading.Thread(target=fa, args=(url,)) for url in urls]

    for t in ths:
        t.start()

    for t in ths:
        t.join()

    ks = list((u, ma[u].strip()[:limit]) for u in ma if ma[u])
    PRE = "*" * 20
    iprint(f"{PRE} f_scan_url good {PRE}")
    for k, v in ks:
        iprint(f"{k} {v}")

    return [k for k, v in ks]


def is_port_open(dst: str, port: int, desc: str = "", timeout: int = 5) -> bool:
    # create a socket object
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    # set a timeout of 1 second
    s.settimeout(timeout)
    try:
        # try to connect to the port
        s.connect((dst, port))
        # port is open
        # iprint(f"Port {port} is open")
        return True
    except:
        # port is closed
        # iprint(f"Port {port} is closed")
        return False
    finally:
        # always close the socket
        s.close()



from dk_imports import *
from dknovautils import *
from dkfiles import *


def oper_():
    pass


def path_eval_1238(
    si: Iterator[str],
    exp: str = None,
    verbose: bool = True,
) -> Iterator[str]:
    """

    abc='mv "{a}" to "{x}"'

    ext = extension (including the dot)
    extnd = extension without the dot

    bn = basename
    bnne = basename without extension

    fp = fullpath
    fpp = fullpath of parent




    """

    def fstr(exp: str):
        assert "'" not in exp, f"err56715 bad quote {exp}"
        return f""" f'{exp}' """.strip()

    for ln in si:
        lnstrip = ln.strip()
        assert lnstrip == ln, f"err72687 {ln}"
        bad1 = ln.endswith("/") or ln.endswith("\\")
        assert not bad1, f"err85854 {ln}"

        dkf = DkFile(ln)

        pif = dkf.path_info

        res = eval(fstr(exp), dataclasses.asdict(pif))

        if verbose:
            iprint(f"yield {res}")
        yield res

    pass


def path_1234(si: Iterator[str], SEP=" && ") -> Iterator[str]:
    """
    假定：
        输入都是普通文件
        全部路径中不包含&&字符

    stdin 输入是每行一个全路径（例如用find生成的）
    stdout 输出是 SEP分割的内容 分隔符两端无空格
    分别是 100&&ext&&filename&&fullpath
    d 取值为d or f 表示是目录或者普通文件
    ext是原始扩展名 包括点符号 比如 .jpg .txt 如果是d 或者 无扩展名 那么是空字符串
    filename是文件名称
    fullpath就是输入的内容

    """
    N = 1234
    S2 = SEP.strip()
    printi = partial(print, flush=True)

    for ln in si:
        assert SEP not in ln, f"err77509 {ln}"
        dkf = DkFile(ln)
        bn = dkf.basename
        ext = dkf.extension[0:]

        out = SEP.join([str(N), ext, bn, ln])
        yield out
        # printi(out)


_demo100 = """


./tb/ta
./Calibre Library
./Calibre Library/(Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui Ta  (Mei )A La Si Tai Er _Shi Mi Si
./Calibre Library/(Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui Ta  (Mei )A La Si Tai Er _Shi Mi Si/Du Cai Zhe Shou Ce 1 (2)
./Calibre Library/(Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui Ta  (Mei )A La Si Tai Er _Shi Mi Si/Du Cai Zhe Shou Ce 1 (2)/Du Cai Zhe Shou Ce 1 - (Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui .pdf
./Calibre Library/(Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui Ta  (Mei )A La Si Tai Er _Shi Mi Si/Du Cai Zhe Shou Ce 1 (2)/cover.jpg
./Calibre Library/(Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui Ta  (Mei )A La Si Tai Er _Shi Mi Si/Du Cai Zhe Shou Ce 1 (2)/Du Cai Zhe Shou Ce 1 - (Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui .epub
./Calibre Library/(Mei )Bu Lu Si _Bu Lu Nuo _De _Mei Si Kui Ta  (Mei )A La Si Tai Er _Shi Mi Si/Du Cai Zhe Shou Ce 1 (2)/metadata.opf
./Calibre Library/metadata.db
./Calibre Library/John Schember
./Calibre Library/John Schember/Quick Start Guide (1)
./Calibre Library/John Schember/Quick Start Guide (1)/cover.jpg
./Calibre Library/John Schember/Quick Start Guide (1)/Quick Start Guide - John Schember.epub
./Calibre Library/John Schember/Quick Start Guide (1)/metadata.opf
./Calibre Library/metadata_db_prefs_backup.json
./.android
./.android/debug.keystore.lock
./.android/analytics.settings
./.android/adbkey
./.android/modem-nv-ram-5554
./.android/avd
./.android/avd/Pixel_Fold_API_35.avd
./.android/avd/Pixel_Fold_API_35.avd/config.ini
./.android/avd/Pixel_Fold_API_35.avd/cache.img
./.android/avd/Pixel_Fold_API_35.avd/sdcard.img.qcow2
./.android/avd/Pixel_Fold_API_35.avd/read-snapshot.txt
./.android/avd/Pixel_Fold_API_35.avd/encryptionkey.img
./.android/avd/Pixel_Fold_API_35.avd/multiinstance.lock
./.android/avd/Pixel_Fold_API_35.avd/sdcard.img
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator/iccprofile_for_sim0.xml
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator/modem_nvram.json
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator/iccprofile_for_carrierapitests.xml
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator/etc
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator/etc/modem_simulator
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator/etc/modem_simulator/files
./.android/avd/Pixel_Fold_API_35.avd/modem_simulator/etc/modem_simulator/files/numeric_operator.xml
./.android/avd/Pixel_Fold_API_35.avd/hardware-qemu.ini
./.android/avd/Pixel_Fold_API_35.avd/version_num.cache



"""


demo_110 = """


./NCE.01.Parts.III.01.mp3
./NCE.02.Parts.III.02.mp3
./NCE.03.Parts.III.03.mp3
./NCE.04.Parts.III.04.mp3
./NCE.05.Parts.III.05.mp3
./NCE.06.Parts.III.06.mp3
./NCE.07.Parts.IV.01.mp3
./NCE.08.Parts.IV.02.mp3
./NCE.09.Parts.IV.03.mp3
./NCE.10.Parts.IV.04.mp3
./NCE.11.Parts.IV.05.mp3



"""


"""

cd /mnt/w_hot/_DISC_B_BAK/HomeShareEngllish/EnglishTextBooks
find ~+ -type f -name "*.mp3" | grep -i -v track > ~/files.txt


"""
exp_ffmpeg_mp3_to_mp4 = """ ffmpeg -i "{fpp}/{bn}" "{fpp}/{bn}.mp4" """


if __name__ == "__main__":

    # path_1234(_demo100.strip().splitlines())

    # exp = 'mv "{fpp}/{bn}" to "{fpp}/{bnne}.mp4"'

    input_str = pyperclip.paste()
    print("input:")
    print(input_str)

    input_str = input_str.strip().splitlines()
    exp = exp_ffmpeg_mp3_to_mp4.strip()
    rss = [ln for ln in path_eval_1238(input_str, exp)]

    rss = "\n".join(rss)
    print("output:")
    print(rss)
    
    

    pyperclip.copy(rss)

    pass



from __future__ import annotations

from dknovautils.commons import *

import pathlib


_debug = False


@dataclass
class DkPathInfo:
    ext: str
    extnd: str
    bn: str
    bnne: str
    fp: str
    fpp: str


"""

        dkf = DkFile(ln)



        namespace = {
            "ext": ext,
            "extnd": extnd,
            "bn": bn,
            "bnne": bnne,
            "fp": fp,
            "fpp": fpp,
        }


"""


class DkFile(object):

    def __init__(self, pathstr: str) -> None:
        """
        增加一个参数 将路径中的分隔符转化为 slash or back-slash or keep-original.

        """
        self.pathstr: str = str(pathstr)

        self.path: Path = Path(pathstr)

    def __key(self) -> Any:
        return self.path

    def __hash__(self) -> int:
        return hash(self.__key())

    def __eq__(self, other: Any) -> Any:
        if isinstance(other, DkFile):
            return self.__key() == other.__key()
        return NotImplemented

    @property
    def basename(self) -> str:
        return os.path.basename(self.path)


    def basename_without_extension(self) -> str:
        bn: str = self.basename
        return bn[: (len(bn) - len(self.extension))]

    # @property
    # def name_meta(self) -> str:

    #     return os.path.basename(self.path)

    @property
    def path_info(self) -> DkPathInfo:
        dkf = self
        ln = self.pathstr

        ext = dkf.extension[0:]
        extnd = ext[1:]

        bn = dkf.basename
        bnne = bn[: len(bn) - len(ext)]

        fp = ln
        fpp = ln[: len(fp) - len(bn) - 1]

        return DkPathInfo(
            ext=ext,
            extnd=extnd,
            bn=bn,
            bnne=bnne,
            fp=fp,
            fpp=fpp,
        )

    @DeprecationWarning
    @property
    def filesize(self) -> int:
        return self.file_size()  # getsize(self.path)

    def file_size(self) -> int:
        return getsize(self.path)

    @property
    def dirname(self) -> str:
        return os.path.dirname(self.path)

    @property
    def extension(self) -> str:
        """包括点号 比如 .txt 如果没有扩展名 则返回空串"""
        # function to return the file extension
        file_extension = self.path.suffix
        return file_extension

    @property
    def extensionlower(self) -> str:
        """包括点号 比如 .txt 如果没有扩展名 则返回空串"""
        return self.extension.lower()

    def get_mtime_millis(self) -> int:
        r = int(self.get_mtime() * 1000)
        return r

    def get_mtime(self) -> float:
        """
        epoch seconds

        https://docs.python.org/3/library/os.path.html#os.path.getmtime

        Return the time of last modification of path. The return value is a floating-point number giving the number of seconds since the epoch (see the time module). Raise OSError if the file does not exist or is inaccessible.

        https://stackoverflow.com/questions/237079/how-do-i-get-file-creation-and-modification-date-times


        import os
        import datetime
        def modification_date(filename):
            t = os.path.getmtime(filename)
            return datetime.datetime.fromtimestamp(t)

        import os, time
        (mode, ino, dev, nlink, uid, gid, size, atime, mtime, ctime) = os.stat(file)
        print("last modified: %s" % time.ctime(mtime))


        """
        mt = os.path.getmtime(self.path)
        return mt

    def remove_curr_file(self):
        """
        Remove (delete) the file path. If path is a directory, an OSError is raised. Use rmdir() to remove directories. If the file does not exist, a FileNotFoundError is raised.

        """
        assert not self.is_dir()
        os.remove(self.path)

    def remove_curr_dir(self):
        assert self.is_dir()
        os.rmdir(self.path)

    def parent_dkfile(self) -> DkFile:
        r = DkFile(str(self.path.parent))
        return r

    @DeprecationWarning
    def exists(self) -> bool:
        return self.is_exists()

    def is_exists(self) -> bool:
        return self.path.exists()

    def is_file(self) -> bool:
        return self.path.is_file()

    def is_dir(self) -> bool:
        return self.path.is_dir()

    def __repr__(self) -> str:
        return self.__str__()

    def __str__(self) -> str:
        return str(self.path)

    @staticmethod
    def clear_dir(folder: str) -> None:
        """

        https://stackoverflow.com/questions/185936/how-to-delete-the-contents-of-a-folder

        """
        import os, shutil

        for filename in os.listdir(folder):
            file_path = os.path.join(folder, filename)
            try:
                if os.path.isfile(file_path) or os.path.islink(file_path):
                    os.unlink(file_path)
                elif os.path.isdir(file_path):
                    shutil.rmtree(file_path)
            except Exception as e:
                iprint("Failed to delete %s. Reason: %s" % (file_path, e))

    @staticmethod
    def file_md5(
        f: str,
        *,
        md5Cache: Dict[str, str] | None = None,
        cb: Callable[[bytes], None] = None,
    ) -> str:
        """ """

        if md5Cache is None:
            md5Cache = {}

        if f in md5Cache:
            r = md5Cache[f]
            assert len(r) == 32
            return r
        else:
            dkf = DkFile(f)
            iprint_debug(f"gen md5 begin. size:{dkf.file_size():,} {f}")
            r = DkFile._fhash(dkf, hashlib.md5(), cb=cb)
            md5Cache[f] = r
            return r

    @staticmethod
    def _fhash(
        dkf: DkFile,
        _fsh,
        *,
        cb: Callable[[bytes], None] = None,
    ):
        size_1g = 1024 * 1024 * 1024
        size_8k = 8192
        LN = size_1g // size_8k
        readn = 1
        bs = 0
        allsize = dkf.file_size()
        """
        file_hash = _fsh  # hashlib.sha1()
            while chunk := f.read(size_8k):
                file_hash.update(chunk)
                r = file_hash.hexdigest().lower()
        
        """

        file_hash = _fsh  # hashlib.sha1()
        with open(dkf.path, "rb", buffering=1024 * 1024) as f:
            while chunk := f.read(size_8k):
                file_hash.update(chunk)
                if cb:
                    cb(chunk)
                readn += 1
                bs += len(chunk)
                if readn % LN == 0:
                    iprint_debug(f"gen hash. read:{bs:,} total:{allsize:,}")

        r = file_hash.hexdigest().lower()
        assert len(r) == 32
        return r

    @staticmethod
    def file_sha1(f: str) -> str:
        import hashlib

        dkf = DkFile(f)
        iprint_debug(f"gen sha1 begin. size:{dkf.file_size():,} {f}")
        r = DkFile._fhash(dkf, hashlib.sha1())
        return r

    @classmethod
    def list_dir(cls, d: str) -> List[DkFile]:
        """仅仅列举目录d的下一级子元素（不包括更深的子元素）"""
        fs = [DkFile(join(d, f)) for f in os.listdir(d)]
        # fs=[DkFile(f) for f in fs]
        return fs

    @staticmethod
    def fill_disk_simple(
        *,
        dir: str = ".",
        verify: bool = True,
        size: int = 1024 * 1024 * 100,
        pre: str = "tmp",
        verbose: bool = False,
    ) -> None:
        import numpy as np

        """
        在目录下创建文件. 文件大小约为10M. 创建之后马上读出检查内容是否正确. 用来对硬盘进行简单的检查.
        直到写满磁盘为止, [Errno 28] No space left on device 最后退出.
        
        mkdir tmp
        python3.10 -c "from dknovautils import *; DkFile.fill_disk_simple(dir='./tmp',verify=True,verbose=True)" |& tee -a ~/fill.log

        """
        bse = b"a"
        bs = np.frombuffer(bse * (1024 * 1024), dtype=np.uint8)

        if isinstance(size, int):
            assert size >= 1024 * 1024
            N = size // len(bs)
        else:
            raise Exception("unimplemented")

        def fpath(fid: int) -> Path:
            return Path(f"{dir}/{pre}{fid:08d}")

        iprint("begin write file")
        fid = 1
        while True:
            file = fpath(fid)
            if verbose:
                iprint(f"write file {file}")
            try:
                with file.open(mode="wb") as f:
                    for i in range(N):
                        f.write(bs)
                    f.flush()
                fid += 1
            except Exception as e:
                iprint(f"error {file} {e}")
                break

        iprint("fill disk end")

        if verify:
            DkFile.verify_disk_simple(dir=dir, pre=pre, verbose=verbose)

    @staticmethod
    def verify_disk_simple(
        *,
        dir: str = ".",
        pre: str = "tmp",
        verbose: bool = False,
    ) -> None:
        import numpy as np

        """
        在目录下创建文件. 文件大小约为10M. 
        直到写满磁盘为止, [Errno 28] No space left on device 最后退出.
        
        注意,下面命令的日志文件不要记录在当前磁盘中.因为当前磁盘会充满,会无法写入日志.
        mkdir tmp
        python3.10 -c "from dknovautils import *; DkFile.verify_disk_simple(dir='./tmp',verbose=True)" |& tee -a ~/verify.log

        """
        bse = b"a"
        bs = np.frombuffer(bse * (1024 * 1024), dtype=np.uint8)

        iprint("begin verify")

        files = (f for f in DkFile.listdir(dir) if f.basename.startswith(pre))
        for idx, file in enumerate(files):
            file = file.path
            if verbose:
                iprint(f"verify file {idx} {file}")
            try:
                with file.open(mode="rb") as f:
                    r = f.read(len(bs))
                    bs2 = np.frombuffer(r, dtype=np.uint8)
                    if len(bs2) == len(bs):
                        assert np.all(bs2 == bs), f"err56373 verify error {file}"
                    else:
                        condition = bs2 == bs[0]
                        assert np.all(condition), f"err56374 verify error {file}"
                # fid += 1
            except Exception as e:
                iprint(f"error idx{idx} {file} {e}")
                break

        iprint("verify disk end")


"""




"""


class DkPyFiles(object):
    pass


if __name__ == "__main__":
    print("OK")



from __future__ import annotations

"""

测试注释1

"""

# 单行注释1

from dknovautils import *

# from dknovautils.commons import *
# from dknovautils.dkfiles import DkPyFiles
# from dknovautils.dkat import AT

# from dknovautils.dkipy import DkIpyUtils

# from commons import *
# from dkfiles import DkPyFiles
# from dkat import AT

print("test01 start")

"""

测试注释2

"""

"""

测试注释3

"""


def test_d_001():
    """

    测试注释4

    What is a Circular Import?

    """

    assert 1 == 1, "err5162"

    assert str(None) == "None", str(None)

    assert AT.VERSION >= "0.0.39", f"err5484 {AT.VERSION}"

    AT.bindport_httpserver(port=10134, daemon=True)

    AT.log_basicConfig()

    AT.mybeep()

    AT.logger.info("hello dkn logger")

    time.sleep(1.0)

    iprint(f"now:{AT.fepochSecs()}")

    print(AT.sdf_isocompact_format_datetime())

    iprint_warn("hello22")
    iprint_info("hello22")
    iprint_error("hello22")

    assert AT._AstWarnCnt_ == 1 and AT._AstErrorCnt_ == 1

    print("=" * 5)

    iprint(AT.VERSION)

    cstr = AT.sdf_logger_format_datetime()
    print(cstr)

    def f_abc(a):
        return a

    # TypeError: cannot set 'f_abc' attribute of immutable type 'str'
    # str.f_abc = f_abc
    # assert "abc".f_abc(123)==123,'err9592'

    # 可以直接修改函数
    DkPyFiles.f_remove_comments_v1 = lambda x: x

    assert DkPyFiles.f_remove_comments_v1(123) == 123, "err2556"

    # now=datetime.now()
    # datetime.faa = 1

    # DkIpyUtils.magic_fc(123)

    assert DkIpyUtils.hello(1) == 1

    run_simple_a("ls")

    if False:

        r = AT.md5_hex(b"abc")
        iprint(f"r: {r}")

        ra = AT.bindport_httpserver(port=8070)
        assert ra

        r = AT.bindport_httpserver(port=8070)
        assert not r

        ra.shutdown()

        x = 0

        def fxx():
            nonlocal x
            x += 1

        r = AT.bindport_httpserver_cb(port=8070, cb=fxx)
        r2 = AT.bindport_httpserver_cb(port=8070, cb=fxx)
        assert r and not r2 and x == 1
        r.shutdown()

        rc, out, err = run_simple_b(r"cmd.exe /c dir e:\etc ", encoding="gbk")
        iprint(out)

    if False:
        run_simple_a("rm /mnt/d/tmp/tmp/etc/* -rf")
        # 如此写法倒是可以勉强运行 但是中文都乱码了
        # robocopy的/unicode输出编码似乎有问题 搭配 'utf-16-le'还是不能正常运行的
        rc, out, err = run_simple_b(
            "robocopy.exe e:/etc d:/tmp/tmp/etc /MIR ",
            encoding="GBK",
            errors="replace",
        )
        iprint(out)

    if False:
        run_simple_a("rm /mnt/d/tmp/tmp/etc2/* -rf")
        # 如此写法似乎是最好的 中文的文件名称编码倒是正常的 只是有少部分中文乱码
        # robocopy的/unicode输出编码似乎也有问题 搭配 'utf-16-le'还是不能正常运行的
        rc, out, err = run_simple_b(
            "robocopy.exe e:/etc2 d:/tmp/tmp/etc2 /MIR ",
            encoding="GBK",
            errors="replace",
        )
        iprint(out)

    if False:
        DkNetwork.net_check_shutdown("http://192.168.0.2")

    if True:
        r = dknetwork.is_port_open("192.168.0.1", port=80)
        assert r
        r = dknetwork.is_port_open("192.168.0.1", port=2222)
        assert not r

    if True:
        dknetwork.http_scan_urls(urls=["http://192.168.0.1"])

    if True:
        dknetwork.f_test_http_get_url_simple_str()

    if False:
        HomeLanUtils.hlan_checkurl_shutdown_bj(ST=60 * 0.5)
        pass


if __name__ == "__main__":
    test_d_001()
    # hlan_checkurl_shutdown_bj()

    print(AT.VERSION)


"""

https://packaging.python.org/en/latest/tutorials/packaging-projects/


"""
